#include "Gadget/Engine.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G;
	G.Window.SetSize(640, 640);
	G.Window.SetClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	G.Window.Open();
	G.Mouse.SetVisibility(true);

	glDisable(GL_DEPTH_TEST); //Ignore depth, we'll render everything from back to front

	//It is common in tile based levels for the world to be rendered in patches or groups 
	//of tiles. That way, we only render the patches that are on screen. Rendering a lot of 
	//sprites that are off screen can cause a game to run slow. 
	Image tileImg;
	tileImg.Load("Images/Tiles.png"); //This image contains all the tiles
	Sprite tileSpr(32.0f, 32.0f); //This same sprite will be used for all the tiles

	//Create a patch of 20 by 20 tiles, 400 tiles in all
	const int patchWidth  = 20;
	const int patchHeight = 20;
	//Place the top left corner of the patch 1/2 a tile width from the top and left side
	Vector2D patchPos(16.0f, float(G.Window.GetHeight())-16.0f);

	//Create a map (normally you would load in a file created by a map editor)
	unsigned char patch[patchWidth*patchHeight] = {
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,2,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,1,0,0,0,0,0,1,1,1,1,1,0,0,1,
		1,0,0,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,
		1,0,0,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,
		1,0,0,0,0,0,1,0,0,0,0,0,1,1,0,1,1,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,1,0,1,1,0,1,1,1,1,1,1,1,0,0,1,1,1,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,1,0,1,0,1,0,0,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,0,3,0,0,0,0,0,0,1,0,0,1,0,1,0,1,0,0,1,
		1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};


	while(G.Window.IsOpen())//loop as long as the window is open
	{
		G.Renderer.BeginFrame(); G.Renderer.Render2D();

			//ESC closes the window
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	

			//Render the patch
			Shader::Blend(2); //threshold blending
			for(int y = 0; y<patchHeight; y++)
			{
				for(int x = 0; x<patchWidth; x++)
				{
					if(patch[x+y*patchWidth]!=0)//don't render anything if the tile is 0
					{
						//Update the sprite
						tileSpr.x = patchPos.x + float(x)*tileSpr.width;
						tileSpr.y = patchPos.y - float(y)*tileSpr.height;
						tileSpr.SelectAnimationFrame(4, 8, patch[x+y*patchWidth]-1);

						//Render a tile
						tileImg.Activate();
						tileSpr.Render();
					}
				}
			}

			Shader::Deactivate(); //Turns off any shaders or blending
			
		G.Renderer.EndFrame();	
	}
	return 0;
}			